@@ -1,15 +1,15 @@
-import os
-import numpy as np
-import numba
 from collections import defaultdict
-from elastica.wrappers import *
+from elastica.modules import *
 from elastica.rod.cosserat_rod import *
+from elastica.rod.knot_theory import *
 from elastica.rigidbody import *
 from elastica.boundary_conditions import *
 from elastica.external_forces import *
 from elastica.callback_functions import *
+from elastica.dissipation import *
 from elastica.interaction import *
 from elastica.joint import *
 from elastica.timestepper import *
 from elastica.restart import *
 from elastica.reset_functions_for_block_structure import *
+from elastica.typing import *
