@@ -6,6 +6,7 @@ __all__ = [
     "UniformTorques",
     "UniformForces",
     "MuscleTorques",
+    "EndpointForcesSinusoidal",
 ]
 
 import numpy as np
@@ -131,24 +132,24 @@ class EndpointForces(NoForces):
         Attributes
         ----------
         start_force: numpy.ndarray
-            2D (dim, 1) array containing data with 'float' type. Force applied to first node of the rod-like object.
+            1D (dim) array containing data with 'float' type. Force applied to first node of the rod-like object.
         end_force: numpy.ndarray
-            2D (dim, 1) array containing data with 'float' type. Force applied to last node of the rod-like object.
+            1D (dim) array containing data with 'float' type. Force applied to last node of the rod-like object.
         ramp_up_time: float
             Applied forces are ramped up until ramp up time.
 
     """
 
-    def __init__(self, start_force, end_force, ramp_up_time=0.0):
+    def __init__(self, start_force, end_force, ramp_up_time):
         """
 
         Parameters
         ----------
         start_force: numpy.ndarray
-            2D (dim, 1) array containing data with 'float' type.
+            1D (dim) array containing data with 'float' type.
             Force applied to first node of the rod-like object.
         end_force: numpy.ndarray
-            2D (dim, 1) array containing data with 'float' type.
+            1D (dim) array containing data with 'float' type.
             Force applied to last node of the rod-like object.
         ramp_up_time: float
             Applied forces are ramped up until ramp up time.
@@ -157,15 +158,10 @@ class EndpointForces(NoForces):
         super(EndpointForces, self).__init__()
         self.start_force = start_force
         self.end_force = end_force
-        assert ramp_up_time >= 0.0
+        assert ramp_up_time > 0.0
         self.ramp_up_time = ramp_up_time
 
     def apply_forces(self, system, time=0.0):
-        # factor = min(1.0, time / self.ramp_up_time)
-        #
-        # system.external_forces[..., 0] += self.start_force * factor
-        # system.external_forces[..., -1] += self.end_force * factor
-
         self.compute_end_point_forces(
             system.external_forces,
             self.start_force,
@@ -187,9 +183,9 @@ class EndpointForces(NoForces):
         external_forces: numpy.ndarray
             2D (dim, blocksize) array containing data with 'float' type. External force vector.
         start_force: numpy.ndarray
-            2D (dim, 1) array containing data with 'float' type.
+            1D (dim) array containing data with 'float' type.
         end_force: numpy.ndarray
-            2D (dim, 1) array containing data with 'float' type.
+            1D (dim) array containing data with 'float' type.
             Force applied to last node of the rod-like object.
         time: float
         ramp_up_time: float
@@ -307,7 +303,7 @@ class MuscleTorques(NoForces):
         phase_shift,
         direction,
         rest_lengths,
-        ramp_up_time=0.0,
+        ramp_up_time,
         with_spline=False,
     ):
         """
@@ -340,7 +336,7 @@ class MuscleTorques(NoForces):
         self.wave_number = wave_number
         self.phase_shift = phase_shift
 
-        assert ramp_up_time >= 0.0
+        assert ramp_up_time > 0.0
         self.ramp_up_time = ramp_up_time
 
         # s is the position of nodes on the rod, we go from node=1 to node=nelem-1, because there is no
@@ -475,3 +471,111 @@ def inplace_substraction(external_force_or_torque, force_or_torque):
     for i in range(3):
         for k in range(blocksize):
             external_force_or_torque[i, k] -= force_or_torque[i, k]
+
+
+class EndpointForcesSinusoidal(NoForces):
+    """
+    This class applies sinusoidally varying forces to the ends of a rod.
+    Forces are applied in a plane, which is defined by the tangent_direction and normal_direction.
+
+        Attributes
+        ----------
+        start_force_mag: float
+            Magnitude of the force that is applied to the start of the rod (node 0).
+        end_force_mag: float
+            Magnitude of the force that is applied to the end of the rod (node -1).
+        ramp_up_time: float
+            Applied forces are applied in the normal direction until time reaches ramp_up_time.
+        normal_direction: np.ndarray
+            An array (3,) contains type float.
+            This is the normal direction of the rod.
+        roll_direction: np.ndarray
+            An array (3,) contains type float.
+            This is the direction perpendicular to rod tangent, and rod normal.
+
+        Notes
+        -----
+        In order to see example how to use this class, see joint examples.
+
+    """
+
+    def __init__(
+        self,
+        start_force_mag,
+        end_force_mag,
+        ramp_up_time=0.0,
+        tangent_direction=np.array([0, 0, 1]),
+        normal_direction=np.array([0, 1, 0]),
+    ):
+        """
+
+        Parameters
+        ----------
+        start_force_mag: float
+            Magnitude of the force that is applied to the start of the rod (node 0).
+        end_force_mag: float
+            Magnitude of the force that is applied to the end of the rod (node -1).
+        ramp_up_time: float
+            Applied muscle torques are ramped up until ramp up time.
+        tangent_direction: np.ndarray
+            An array (3,) contains type float.
+            This is the tangent direction of the rod, or normal of the plane that forces applied.
+        normal_direction: np.ndarray
+            An array (3,) contains type float.
+            This is the normal direction of the rod.
+        """
+        super(EndpointForcesSinusoidal, self).__init__()
+        # Start force
+        self.start_force_mag = start_force_mag
+        self.end_force_mag = end_force_mag
+
+        # Applied force directions
+        self.normal_direction = normal_direction
+        self.roll_direction = np.cross(normal_direction, tangent_direction)
+
+        assert ramp_up_time >= 0.0
+        self.ramp_up_time = ramp_up_time
+
+    def apply_forces(self, system, time=0.0):
+
+        if time < self.ramp_up_time:
+            # When time smaller than ramp up time apply the force in normal direction
+            # First pull the rod upward or downward direction some time.
+            start_force = -2.0 * self.start_force_mag * self.normal_direction
+            end_force = -2.0 * self.end_force_mag * self.normal_direction
+
+            system.external_forces[..., 0] += start_force
+            system.external_forces[..., -1] += end_force
+
+        else:
+            # When time is greater than ramp up time, forces are applied in normal
+            # and roll direction or forces are in a plane perpendicular to the
+            # direction.
+
+            # First force applied to start of the rod
+            roll_forces_start = (
+                self.start_force_mag
+                * np.cos(0.5 * np.pi * (time - self.ramp_up_time))
+                * self.roll_direction
+            )
+            normal_forces_start = (
+                self.start_force_mag
+                * np.sin(0.5 * np.pi * (time - self.ramp_up_time))
+                * self.normal_direction
+            )
+            start_force = roll_forces_start + normal_forces_start
+            # Now force applied to end of the rod
+            roll_forces_end = (
+                self.end_force_mag
+                * np.cos(0.5 * np.pi * (time - self.ramp_up_time))
+                * self.roll_direction
+            )
+            normal_forces_end = (
+                self.end_force_mag
+                * np.sin(0.5 * np.pi * (time - self.ramp_up_time))
+                * self.normal_direction
+            )
+            end_force = roll_forces_end + normal_forces_end
+            # Update external forces
+            system.external_forces[..., 0] += start_force
+            system.external_forces[..., -1] += end_force
