@@ -129,17 +129,26 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
 
         # Things in nodes that are scalars
         #             0 ("mass", float64[:]),
-        map_scalar_dofs_in_rod_nodes = {"mass": 0}
+        map_scalar_dofs_in_rod_nodes = {
+            "mass": 0,
+            "dissipation_constant_for_forces": 1,
+        }
         self.scalar_dofs_in_rod_nodes = np.zeros(
             (len(map_scalar_dofs_in_rod_nodes), self.n_nodes)
         )
-        self.mass = self.scalar_dofs_in_rod_nodes[0]
-        for system_idx, system in enumerate(systems):
-            start_idx = self.start_idx_in_rod_nodes[system_idx]
-            end_idx = self.end_idx_in_rod_nodes[system_idx]
-            self.mass[start_idx:end_idx] = system.mass.copy()
-            # create a view back to the rod after copying variable into the block structure
-            system.mass = np.ndarray.view(self.mass[start_idx:end_idx])
+        for k, v in map_scalar_dofs_in_rod_nodes.items():
+            self.__dict__[k] = np.lib.stride_tricks.as_strided(
+                self.scalar_dofs_in_rod_nodes[v], (self.n_nodes,)
+            )
+
+        for k, v in map_scalar_dofs_in_rod_nodes.items():
+            for system_idx, system in enumerate(systems):
+                start_idx = self.start_idx_in_rod_nodes[system_idx]
+                end_idx = self.end_idx_in_rod_nodes[system_idx]
+                self.__dict__[k][..., start_idx:end_idx] = system.__dict__[k].copy()
+                system.__dict__[k] = np.ndarray.view(
+                    self.__dict__[k][..., start_idx:end_idx]
+                )
 
         # Things in nodes that are vectors
         #             0 ("position_collection", float64[:, :]),
@@ -206,8 +215,7 @@ class MemoryBlockCosseratRod(CosseratRod, _RodSymplecticStepperMixin):
             "rest_lengths": 4,
             "dilatation": 5,
             "dilatation_rate": 6,
-            "dissipation_constant_for_forces": 7,
-            "dissipation_constant_for_torques": 8,
+            "dissipation_constant_for_torques": 7,
         }
         self.scalar_dofs_in_rod_elems = np.zeros(
             (len(map_scalar_dofs_in_rod_elems), self.n_elems)
